from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_PRE_AUTH: Literal["enable", "disable"]
VALID_BODY_EXTERNAL_PRE_AUTH: Literal["enable", "disable"]
VALID_BODY_MESH_BACKHAUL: Literal["enable", "disable"]
VALID_BODY_BROADCAST_SSID: Literal["enable", "disable"]
VALID_BODY_SECURITY: Literal["open", "wep64", "wep128", "wpa-personal", "wpa-enterprise", "wpa-only-personal", "wpa-only-enterprise", "wpa2-only-personal", "wpa2-only-enterprise", "wpa3-enterprise", "wpa3-only-enterprise", "wpa3-enterprise-transition", "wpa3-sae", "wpa3-sae-transition", "owe", "osen"]
VALID_BODY_PMF: Literal["disable", "enable", "optional"]
VALID_BODY_BEACON_PROTECTION: Literal["disable", "enable"]
VALID_BODY_OKC: Literal["disable", "enable"]
VALID_BODY_MBO: Literal["disable", "enable"]
VALID_BODY_MBO_CELL_DATA_CONN_PREF: Literal["excluded", "prefer-not", "prefer-use"]
VALID_BODY_80211K: Literal["disable", "enable"]
VALID_BODY_80211V: Literal["disable", "enable"]
VALID_BODY_NEIGHBOR_REPORT_DUAL_BAND: Literal["disable", "enable"]
VALID_BODY_FAST_BSS_TRANSITION: Literal["disable", "enable"]
VALID_BODY_FT_OVER_DS: Literal["disable", "enable"]
VALID_BODY_SAE_GROUPS: Literal["19", "20", "21"]
VALID_BODY_OWE_GROUPS: Literal["19", "20", "21"]
VALID_BODY_OWE_TRANSITION: Literal["disable", "enable"]
VALID_BODY_ADDITIONAL_AKMS: Literal["akm6", "akm24"]
VALID_BODY_EAPOL_KEY_RETRIES: Literal["disable", "enable"]
VALID_BODY_TKIP_COUNTER_MEASURE: Literal["enable", "disable"]
VALID_BODY_EXTERNAL_WEB_FORMAT: Literal["auto-detect", "no-query-string", "partial-query-string"]
VALID_BODY_MAC_USERNAME_DELIMITER: Literal["hyphen", "single-hyphen", "colon", "none"]
VALID_BODY_MAC_PASSWORD_DELIMITER: Literal["hyphen", "single-hyphen", "colon", "none"]
VALID_BODY_MAC_CALLING_STATION_DELIMITER: Literal["hyphen", "single-hyphen", "colon", "none"]
VALID_BODY_MAC_CALLED_STATION_DELIMITER: Literal["hyphen", "single-hyphen", "colon", "none"]
VALID_BODY_MAC_CASE: Literal["uppercase", "lowercase"]
VALID_BODY_CALLED_STATION_ID_TYPE: Literal["mac", "ip", "apname"]
VALID_BODY_MAC_AUTH_BYPASS: Literal["enable", "disable"]
VALID_BODY_RADIUS_MAC_AUTH: Literal["enable", "disable"]
VALID_BODY_RADIUS_MAC_MPSK_AUTH: Literal["enable", "disable"]
VALID_BODY_AUTH: Literal["radius", "usergroup"]
VALID_BODY_ENCRYPT: Literal["TKIP", "AES", "TKIP-AES"]
VALID_BODY_SAE_H2E_ONLY: Literal["enable", "disable"]
VALID_BODY_SAE_HNP_ONLY: Literal["enable", "disable"]
VALID_BODY_SAE_PK: Literal["enable", "disable"]
VALID_BODY_AKM24_ONLY: Literal["disable", "enable"]
VALID_BODY_NAS_FILTER_RULE: Literal["enable", "disable"]
VALID_BODY_DOMAIN_NAME_STRIPPING: Literal["disable", "enable"]
VALID_BODY_MLO: Literal["disable", "enable"]
VALID_BODY_LOCAL_STANDALONE: Literal["enable", "disable"]
VALID_BODY_LOCAL_STANDALONE_NAT: Literal["enable", "disable"]
VALID_BODY_LOCAL_STANDALONE_DNS: Literal["enable", "disable"]
VALID_BODY_LOCAL_LAN_PARTITION: Literal["enable", "disable"]
VALID_BODY_LOCAL_BRIDGING: Literal["enable", "disable"]
VALID_BODY_LOCAL_LAN: Literal["allow", "deny"]
VALID_BODY_LOCAL_AUTHENTICATION: Literal["enable", "disable"]
VALID_BODY_CAPTIVE_PORTAL: Literal["enable", "disable"]
VALID_BODY_CAPTIVE_NETWORK_ASSISTANT_BYPASS: Literal["enable", "disable"]
VALID_BODY_PORTAL_TYPE: Literal["auth", "auth+disclaimer", "disclaimer", "email-collect", "cmcc", "cmcc-macauth", "auth-mac", "external-auth", "external-macauth"]
VALID_BODY_INTRA_VAP_PRIVACY: Literal["enable", "disable"]
VALID_BODY_LDPC: Literal["disable", "rx", "tx", "rxtx"]
VALID_BODY_HIGH_EFFICIENCY: Literal["enable", "disable"]
VALID_BODY_TARGET_WAKE_TIME: Literal["enable", "disable"]
VALID_BODY_PORT_MACAUTH: Literal["disable", "radius", "address-group"]
VALID_BODY_BSS_COLOR_PARTIAL: Literal["enable", "disable"]
VALID_BODY_SPLIT_TUNNELING: Literal["enable", "disable"]
VALID_BODY_NAC: Literal["enable", "disable"]
VALID_BODY_VLAN_AUTO: Literal["enable", "disable"]
VALID_BODY_DYNAMIC_VLAN: Literal["enable", "disable"]
VALID_BODY_CAPTIVE_PORTAL_FW_ACCOUNTING: Literal["enable", "disable"]
VALID_BODY_MULTICAST_RATE: Literal["0", "6000", "12000", "24000"]
VALID_BODY_MULTICAST_ENHANCE: Literal["enable", "disable"]
VALID_BODY_IGMP_SNOOPING: Literal["enable", "disable"]
VALID_BODY_DHCP_ADDRESS_ENFORCEMENT: Literal["enable", "disable"]
VALID_BODY_BROADCAST_SUPPRESSION: Literal["dhcp-up", "dhcp-down", "dhcp-starvation", "dhcp-ucast", "arp-known", "arp-unknown", "arp-reply", "arp-poison", "arp-proxy", "netbios-ns", "netbios-ds", "ipv6", "all-other-mc", "all-other-bc"]
VALID_BODY_IPV6_RULES: Literal["drop-icmp6ra", "drop-icmp6rs", "drop-llmnr6", "drop-icmp6mld2", "drop-dhcp6s", "drop-dhcp6c", "ndp-proxy", "drop-ns-dad", "drop-ns-nondad"]
VALID_BODY_MU_MIMO: Literal["enable", "disable"]
VALID_BODY_PROBE_RESP_SUPPRESSION: Literal["enable", "disable"]
VALID_BODY_RADIO_SENSITIVITY: Literal["enable", "disable"]
VALID_BODY_QUARANTINE: Literal["enable", "disable"]
VALID_BODY_VLAN_POOLING: Literal["wtp-group", "round-robin", "hash", "disable"]
VALID_BODY_DHCP_OPTION43_INSERTION: Literal["enable", "disable"]
VALID_BODY_DHCP_OPTION82_INSERTION: Literal["enable", "disable"]
VALID_BODY_DHCP_OPTION82_CIRCUIT_ID_INSERTION: Literal["style-1", "style-2", "style-3", "disable"]
VALID_BODY_DHCP_OPTION82_REMOTE_ID_INSERTION: Literal["style-1", "disable"]
VALID_BODY_PTK_REKEY: Literal["enable", "disable"]
VALID_BODY_GTK_REKEY: Literal["enable", "disable"]
VALID_BODY_EAP_REAUTH: Literal["enable", "disable"]
VALID_BODY_ROAMING_ACCT_INTERIM_UPDATE: Literal["enable", "disable"]
VALID_BODY_RATES_11A: Literal["6", "6-basic", "9", "9-basic", "12", "12-basic", "18", "18-basic", "24", "24-basic", "36", "36-basic", "48", "48-basic", "54", "54-basic"]
VALID_BODY_RATES_11BG: Literal["1", "1-basic", "2", "2-basic", "5.5", "5.5-basic", "11", "11-basic", "6", "6-basic", "9", "9-basic", "12", "12-basic", "18", "18-basic", "24", "24-basic", "36", "36-basic", "48", "48-basic", "54", "54-basic"]
VALID_BODY_RATES_11N_SS12: Literal["mcs0/1", "mcs1/1", "mcs2/1", "mcs3/1", "mcs4/1", "mcs5/1", "mcs6/1", "mcs7/1", "mcs8/2", "mcs9/2", "mcs10/2", "mcs11/2", "mcs12/2", "mcs13/2", "mcs14/2", "mcs15/2"]
VALID_BODY_RATES_11N_SS34: Literal["mcs16/3", "mcs17/3", "mcs18/3", "mcs19/3", "mcs20/3", "mcs21/3", "mcs22/3", "mcs23/3", "mcs24/4", "mcs25/4", "mcs26/4", "mcs27/4", "mcs28/4", "mcs29/4", "mcs30/4", "mcs31/4"]
VALID_BODY_UTM_STATUS: Literal["enable", "disable"]
VALID_BODY_UTM_LOG: Literal["enable", "disable"]
VALID_BODY_SCAN_BOTNET_CONNECTIONS: Literal["disable", "monitor", "block"]
VALID_BODY_ADDRESS_GROUP_POLICY: Literal["disable", "allow", "deny"]
VALID_BODY_STICKY_CLIENT_REMOVE: Literal["enable", "disable"]
VALID_BODY_BSTM_DISASSOCIATION_IMMINENT: Literal["enable", "disable"]
VALID_BODY_BEACON_ADVERTISING: Literal["name", "model", "serial-number"]
VALID_BODY_OSEN: Literal["enable", "disable"]
VALID_BODY_APPLICATION_DETECTION_ENGINE: Literal["enable", "disable"]
VALID_BODY_APPLICATION_DSCP_MARKING: Literal["enable", "disable"]
VALID_BODY_L3_ROAMING: Literal["enable", "disable"]
VALID_BODY_L3_ROAMING_MODE: Literal["direct", "indirect"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_PRE_AUTH",
    "VALID_BODY_EXTERNAL_PRE_AUTH",
    "VALID_BODY_MESH_BACKHAUL",
    "VALID_BODY_BROADCAST_SSID",
    "VALID_BODY_SECURITY",
    "VALID_BODY_PMF",
    "VALID_BODY_BEACON_PROTECTION",
    "VALID_BODY_OKC",
    "VALID_BODY_MBO",
    "VALID_BODY_MBO_CELL_DATA_CONN_PREF",
    "VALID_BODY_80211K",
    "VALID_BODY_80211V",
    "VALID_BODY_NEIGHBOR_REPORT_DUAL_BAND",
    "VALID_BODY_FAST_BSS_TRANSITION",
    "VALID_BODY_FT_OVER_DS",
    "VALID_BODY_SAE_GROUPS",
    "VALID_BODY_OWE_GROUPS",
    "VALID_BODY_OWE_TRANSITION",
    "VALID_BODY_ADDITIONAL_AKMS",
    "VALID_BODY_EAPOL_KEY_RETRIES",
    "VALID_BODY_TKIP_COUNTER_MEASURE",
    "VALID_BODY_EXTERNAL_WEB_FORMAT",
    "VALID_BODY_MAC_USERNAME_DELIMITER",
    "VALID_BODY_MAC_PASSWORD_DELIMITER",
    "VALID_BODY_MAC_CALLING_STATION_DELIMITER",
    "VALID_BODY_MAC_CALLED_STATION_DELIMITER",
    "VALID_BODY_MAC_CASE",
    "VALID_BODY_CALLED_STATION_ID_TYPE",
    "VALID_BODY_MAC_AUTH_BYPASS",
    "VALID_BODY_RADIUS_MAC_AUTH",
    "VALID_BODY_RADIUS_MAC_MPSK_AUTH",
    "VALID_BODY_AUTH",
    "VALID_BODY_ENCRYPT",
    "VALID_BODY_SAE_H2E_ONLY",
    "VALID_BODY_SAE_HNP_ONLY",
    "VALID_BODY_SAE_PK",
    "VALID_BODY_AKM24_ONLY",
    "VALID_BODY_NAS_FILTER_RULE",
    "VALID_BODY_DOMAIN_NAME_STRIPPING",
    "VALID_BODY_MLO",
    "VALID_BODY_LOCAL_STANDALONE",
    "VALID_BODY_LOCAL_STANDALONE_NAT",
    "VALID_BODY_LOCAL_STANDALONE_DNS",
    "VALID_BODY_LOCAL_LAN_PARTITION",
    "VALID_BODY_LOCAL_BRIDGING",
    "VALID_BODY_LOCAL_LAN",
    "VALID_BODY_LOCAL_AUTHENTICATION",
    "VALID_BODY_CAPTIVE_PORTAL",
    "VALID_BODY_CAPTIVE_NETWORK_ASSISTANT_BYPASS",
    "VALID_BODY_PORTAL_TYPE",
    "VALID_BODY_INTRA_VAP_PRIVACY",
    "VALID_BODY_LDPC",
    "VALID_BODY_HIGH_EFFICIENCY",
    "VALID_BODY_TARGET_WAKE_TIME",
    "VALID_BODY_PORT_MACAUTH",
    "VALID_BODY_BSS_COLOR_PARTIAL",
    "VALID_BODY_SPLIT_TUNNELING",
    "VALID_BODY_NAC",
    "VALID_BODY_VLAN_AUTO",
    "VALID_BODY_DYNAMIC_VLAN",
    "VALID_BODY_CAPTIVE_PORTAL_FW_ACCOUNTING",
    "VALID_BODY_MULTICAST_RATE",
    "VALID_BODY_MULTICAST_ENHANCE",
    "VALID_BODY_IGMP_SNOOPING",
    "VALID_BODY_DHCP_ADDRESS_ENFORCEMENT",
    "VALID_BODY_BROADCAST_SUPPRESSION",
    "VALID_BODY_IPV6_RULES",
    "VALID_BODY_MU_MIMO",
    "VALID_BODY_PROBE_RESP_SUPPRESSION",
    "VALID_BODY_RADIO_SENSITIVITY",
    "VALID_BODY_QUARANTINE",
    "VALID_BODY_VLAN_POOLING",
    "VALID_BODY_DHCP_OPTION43_INSERTION",
    "VALID_BODY_DHCP_OPTION82_INSERTION",
    "VALID_BODY_DHCP_OPTION82_CIRCUIT_ID_INSERTION",
    "VALID_BODY_DHCP_OPTION82_REMOTE_ID_INSERTION",
    "VALID_BODY_PTK_REKEY",
    "VALID_BODY_GTK_REKEY",
    "VALID_BODY_EAP_REAUTH",
    "VALID_BODY_ROAMING_ACCT_INTERIM_UPDATE",
    "VALID_BODY_RATES_11A",
    "VALID_BODY_RATES_11BG",
    "VALID_BODY_RATES_11N_SS12",
    "VALID_BODY_RATES_11N_SS34",
    "VALID_BODY_UTM_STATUS",
    "VALID_BODY_UTM_LOG",
    "VALID_BODY_SCAN_BOTNET_CONNECTIONS",
    "VALID_BODY_ADDRESS_GROUP_POLICY",
    "VALID_BODY_STICKY_CLIENT_REMOVE",
    "VALID_BODY_BSTM_DISASSOCIATION_IMMINENT",
    "VALID_BODY_BEACON_ADVERTISING",
    "VALID_BODY_OSEN",
    "VALID_BODY_APPLICATION_DETECTION_ENGINE",
    "VALID_BODY_APPLICATION_DSCP_MARKING",
    "VALID_BODY_L3_ROAMING",
    "VALID_BODY_L3_ROAMING_MODE",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]


"""
Pydantic Models for CMDB - web_proxy/global_

Runtime validation models for web_proxy/global_ configuration.
Generated from FortiOS schema version unknown.

Example Usage:
    >>> from hfortix_fortios.models.cmdb.web_proxy.global_ import 
    >>>
    >>> # Create with validation
    >>> obj = (
    ...     name="example",
    ... )
    >>>
    >>> # Validation happens automatically
    >>> # ValidationError raised if constraints violated
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Child Table Models
# ============================================================================

class GlobalLearnClientIpSrcaddr(BaseModel):
    """
    Child table model for learn-client-ip-srcaddr.
    
    Source address name (srcaddr or srcaddr6 must be set).
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    name: str = Field(max_length=79, default="", description="Address name.")  # datasource: ['firewall.address.name', 'firewall.addrgrp.name']
class GlobalLearnClientIpSrcaddr6(BaseModel):
    """
    Child table model for learn-client-ip-srcaddr6.
    
    IPv6 Source address name (srcaddr or srcaddr6 must be set).
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    name: str = Field(max_length=79, default="", description="Address name.")  # datasource: ['firewall.address6.name', 'firewall.addrgrp6.name']
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class GlobalModel(BaseModel):
    """
    Pydantic model for web_proxy/global_ configuration.
    
    Configure Web proxy global settings.
    
    Validation Rules:        - ssl_cert: max_length=35 pattern=        - ssl_ca_cert: max_length=35 pattern=        - fast_policy_match: pattern=        - ldap_user_cache: pattern=        - proxy_fqdn: max_length=255 pattern=        - max_request_length: min=2 max=64 pattern=        - max_message_length: min=16 max=256 pattern=        - http2_client_window_size: min=65535 max=2147483647 pattern=        - http2_server_window_size: min=65535 max=2147483647 pattern=        - auth_sign_timeout: min=30 max=3600 pattern=        - strict_web_check: pattern=        - forward_proxy_auth: pattern=        - forward_server_affinity_timeout: min=6 max=60 pattern=        - max_waf_body_cache_length: min=1 max=1024 pattern=        - webproxy_profile: max_length=63 pattern=        - learn_client_ip: pattern=        - always_learn_client_ip: pattern=        - learn_client_ip_from_header: pattern=        - learn_client_ip_srcaddr: pattern=        - learn_client_ip_srcaddr6: pattern=        - src_affinity_exempt_addr: pattern=        - src_affinity_exempt_addr6: pattern=        - policy_partial_match: pattern=        - log_policy_pending: pattern=        - log_forward_server: pattern=        - log_app_id: pattern=        - proxy_transparent_cert_inspection: pattern=        - request_obs_fold: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    ssl_cert: str | None = Field(max_length=35, default="Fortinet_Factory", description="SSL certificate for SSL interception.")  # datasource: ['vpn.certificate.local.name']    
    ssl_ca_cert: str | None = Field(max_length=35, default="Fortinet_CA_SSL", description="SSL CA certificate for SSL interception.")  # datasource: ['vpn.certificate.local.name', 'vpn.certificate.hsm-local.name']    
    fast_policy_match: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable fast matching algorithm for explicit and transparent proxy policy.")    
    ldap_user_cache: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable LDAP user cache for explicit and transparent proxy user.")    
    proxy_fqdn: str = Field(max_length=255, default="default.fqdn", description="Fully Qualified Domain Name of the explicit web proxy (default = default.fqdn) that clients connect to.")    
    max_request_length: int | None = Field(ge=2, le=64, default=8, description="Maximum length of HTTP request line (2 - 64 Kbytes, default = 8).")    
    max_message_length: int | None = Field(ge=16, le=256, default=32, description="Maximum length of HTTP message, not including body (16 - 256 Kbytes, default = 32).")    
    http2_client_window_size: int | None = Field(ge=65535, le=2147483647, default=1048576, description="HTTP/2 client initial window size in bytes (65535 - 2147483647, default = 1048576 (1MB)).")    
    http2_server_window_size: int | None = Field(ge=65535, le=2147483647, default=1048576, description="HTTP/2 server initial window size in bytes (65535 - 2147483647, default = 1048576 (1MB)).")    
    auth_sign_timeout: int | None = Field(ge=30, le=3600, default=120, description="Proxy auth query sign timeout in seconds (30 - 3600, default = 120).")    
    strict_web_check: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable strict web checking to block web sites that send incorrect headers that don't conform to HTTP.")    
    forward_proxy_auth: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable forwarding proxy authentication headers.")    
    forward_server_affinity_timeout: int | None = Field(ge=6, le=60, default=30, description="Period of time before the source IP's traffic is no longer assigned to the forwarding server (6 - 60 min, default = 30).")    
    max_waf_body_cache_length: int | None = Field(ge=1, le=1024, default=1, description="Maximum length of HTTP messages processed by Web Application Firewall (WAF) (1 - 1024 Kbytes, default = 1).")    
    webproxy_profile: str | None = Field(max_length=63, default="", description="Name of the web proxy profile to apply when explicit proxy traffic is allowed by default and traffic is accepted that does not match an explicit proxy policy.")  # datasource: ['web-proxy.profile.name']    
    learn_client_ip: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable learning the client's IP address from headers.")    
    always_learn_client_ip: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable learning the client's IP address from headers for every request.")    
    learn_client_ip_from_header: list[LearnClientIpFromHeader] = Field(default="", description="Learn client IP address from the specified headers.")    
    learn_client_ip_srcaddr: list[LearnClientIpSrcaddr] = Field(default=None, description="Source address name (srcaddr or srcaddr6 must be set).")    
    learn_client_ip_srcaddr6: list[LearnClientIpSrcaddr6] = Field(default=None, description="IPv6 Source address name (srcaddr or srcaddr6 must be set).")    
    src_affinity_exempt_addr: list[SrcAffinityExemptAddr] = Field(default="", description="IPv4 source addresses to exempt proxy affinity.")    
    src_affinity_exempt_addr6: list[SrcAffinityExemptAddr6] = Field(default="", description="IPv6 source addresses to exempt proxy affinity.")    
    policy_partial_match: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable policy partial matching.")    
    log_policy_pending: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logging sessions that are pending on policy matching.")    
    log_forward_server: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable forward server name logging in forward traffic log.")    
    log_app_id: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable always log application type in traffic log.")    
    proxy_transparent_cert_inspection: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable transparent proxy certificate inspection.")    
    request_obs_fold: Literal["replace-with-sp", "block", "keep"] | None = Field(default="keep", description="Action when HTTP/1.x request header contains obs-fold (default = keep).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('ssl_cert')
    @classmethod
    def validate_ssl_cert(cls, v: Any) -> Any:
        """
        Validate ssl_cert field.
        
        Datasource: ['vpn.certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('ssl_ca_cert')
    @classmethod
    def validate_ssl_ca_cert(cls, v: Any) -> Any:
        """
        Validate ssl_ca_cert field.
        
        Datasource: ['vpn.certificate.local.name', 'vpn.certificate.hsm-local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('webproxy_profile')
    @classmethod
    def validate_webproxy_profile(cls, v: Any) -> Any:
        """
        Validate webproxy_profile field.
        
        Datasource: ['web-proxy.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_ssl_cert_references(self, client: Any) -> list[str]:
        """
        Validate ssl_cert references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = GlobalModel(
            ...     ssl_cert="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ssl_cert_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.web_proxy.global_.post(policy.to_fortios_dict())
        """
        errors = []
        
        # Validate scalar field
        value = getattr(self, "ssl_cert", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Ssl-Cert '{value}' not found in "
                "vpn/certificate/local"
            )        
        return errors    
    async def validate_ssl_ca_cert_references(self, client: Any) -> list[str]:
        """
        Validate ssl_ca_cert references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/local        - vpn/certificate/hsm-local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = GlobalModel(
            ...     ssl_ca_cert="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ssl_ca_cert_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.web_proxy.global_.post(policy.to_fortios_dict())
        """
        errors = []
        
        # Validate scalar field
        value = getattr(self, "ssl_ca_cert", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.local.exists(value):
            found = True
        elif await client.api.cmdb.vpn.certificate.hsm-local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Ssl-Ca-Cert '{value}' not found in "
                "vpn/certificate/local or vpn/certificate/hsm-local"
            )        
        return errors    
    async def validate_webproxy_profile_references(self, client: Any) -> list[str]:
        """
        Validate webproxy_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - web-proxy/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = GlobalModel(
            ...     webproxy_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_webproxy_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.web_proxy.global_.post(policy.to_fortios_dict())
        """
        errors = []
        
        # Validate scalar field
        value = getattr(self, "webproxy_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.web-proxy.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Webproxy-Profile '{value}' not found in "
                "web-proxy/profile"
            )        
        return errors    
    async def validate_learn_client_ip_srcaddr_references(self, client: Any) -> list[str]:
        """
        Validate learn_client_ip_srcaddr references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/address        - firewall/addrgrp        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = GlobalModel(
            ...     learn_client_ip_srcaddr=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_learn_client_ip_srcaddr_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.web_proxy.global_.post(policy.to_fortios_dict())
        """
        errors = []
        
        # Validate child table items
        values = getattr(self, "learn_client_ip_srcaddr", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.firewall.address.exists(value):
                found = True
            elif await client.api.cmdb.firewall.addrgrp.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Learn-Client-Ip-Srcaddr '{value}' not found in "
                    "firewall/address or firewall/addrgrp"
                )        
        return errors    
    async def validate_learn_client_ip_srcaddr6_references(self, client: Any) -> list[str]:
        """
        Validate learn_client_ip_srcaddr6 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/address6        - firewall/addrgrp6        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = GlobalModel(
            ...     learn_client_ip_srcaddr6=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_learn_client_ip_srcaddr6_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.web_proxy.global_.post(policy.to_fortios_dict())
        """
        errors = []
        
        # Validate child table items
        values = getattr(self, "learn_client_ip_srcaddr6", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.firewall.address6.exists(value):
                found = True
            elif await client.api.cmdb.firewall.addrgrp6.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Learn-Client-Ip-Srcaddr6 '{value}' not found in "
                    "firewall/address6 or firewall/addrgrp6"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_ssl_cert_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_ssl_ca_cert_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_webproxy_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_learn_client_ip_srcaddr_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_learn_client_ip_srcaddr6_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "GlobalModel",    "GlobalLearnClientIpSrcaddr",    "GlobalLearnClientIpSrcaddr6",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-09T09:47:39.673576Z
# ============================================================================


from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_ACTION_TYPE: Literal["email", "fortiexplorer-notification", "alert", "disable-ssid", "system-actions", "quarantine", "quarantine-forticlient", "quarantine-nsx", "quarantine-fortinac", "ban-ip", "aws-lambda", "azure-function", "google-cloud-function", "alicloud-function", "webhook", "cli-script", "diagnose-script", "regular-expression", "slack-notification", "microsoft-teams-notification"]
VALID_BODY_SYSTEM_ACTION: Literal["reboot", "shutdown", "backup-config"]
VALID_BODY_FORTICARE_EMAIL: Literal["enable", "disable"]
VALID_BODY_AZURE_FUNCTION_AUTHORIZATION: Literal["anonymous", "function", "admin"]
VALID_BODY_ALICLOUD_FUNCTION_AUTHORIZATION: Literal["anonymous", "function"]
VALID_BODY_MESSAGE_TYPE: Literal["text", "json", "form-data"]
VALID_BODY_REPLACEMENT_MESSAGE: Literal["enable", "disable"]
VALID_BODY_PROTOCOL: Literal["http", "https"]
VALID_BODY_METHOD: Literal["post", "put", "get", "patch", "delete"]
VALID_BODY_VERIFY_HOST_CERT: Literal["enable", "disable"]
VALID_BODY_FILE_ONLY: Literal["enable", "disable"]
VALID_BODY_EXECUTE_SECURITY_FABRIC: Literal["enable", "disable"]
VALID_BODY_LOG_DEBUG_PRINT: Literal["enable", "disable"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_ACTION_TYPE",
    "VALID_BODY_SYSTEM_ACTION",
    "VALID_BODY_FORTICARE_EMAIL",
    "VALID_BODY_AZURE_FUNCTION_AUTHORIZATION",
    "VALID_BODY_ALICLOUD_FUNCTION_AUTHORIZATION",
    "VALID_BODY_MESSAGE_TYPE",
    "VALID_BODY_REPLACEMENT_MESSAGE",
    "VALID_BODY_PROTOCOL",
    "VALID_BODY_METHOD",
    "VALID_BODY_VERIFY_HOST_CERT",
    "VALID_BODY_FILE_ONLY",
    "VALID_BODY_EXECUTE_SECURITY_FABRIC",
    "VALID_BODY_LOG_DEBUG_PRINT",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]


"""Validation helpers for router/isis - Auto-generated"""

from typing import Any, TypedDict, NotRequired, Literal

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "is-type": "level-1-2",
    "adv-passive-only": "disable",
    "adv-passive-only6": "disable",
    "auth-mode-l1": "password",
    "auth-mode-l2": "password",
    "auth-keychain-l1": "",
    "auth-keychain-l2": "",
    "auth-sendonly-l1": "disable",
    "auth-sendonly-l2": "disable",
    "ignore-lsp-errors": "disable",
    "lsp-gen-interval-l1": 30,
    "lsp-gen-interval-l2": 30,
    "lsp-refresh-interval": 900,
    "max-lsp-lifetime": 1200,
    "spf-interval-exp-l1": "",
    "spf-interval-exp-l2": "",
    "dynamic-hostname": "disable",
    "adjacency-check": "disable",
    "adjacency-check6": "disable",
    "overload-bit": "disable",
    "overload-bit-suppress": "",
    "overload-bit-on-startup": 0,
    "default-originate": "disable",
    "default-originate6": "disable",
    "metric-style": "narrow",
    "redistribute-l1": "disable",
    "redistribute-l1-list": "",
    "redistribute-l2": "disable",
    "redistribute-l2-list": "",
    "redistribute6-l1": "disable",
    "redistribute6-l1-list": "",
    "redistribute6-l2": "disable",
    "redistribute6-l2-list": "",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "is-type": "option",  # IS type.
    "adv-passive-only": "option",  # Enable/disable IS-IS advertisement of passive interfaces onl
    "adv-passive-only6": "option",  # Enable/disable IPv6 IS-IS advertisement of passive interface
    "auth-mode-l1": "option",  # Level 1 authentication mode.
    "auth-mode-l2": "option",  # Level 2 authentication mode.
    "auth-password-l1": "password",  # Authentication password for level 1 PDUs.
    "auth-password-l2": "password",  # Authentication password for level 2 PDUs.
    "auth-keychain-l1": "string",  # Authentication key-chain for level 1 PDUs.
    "auth-keychain-l2": "string",  # Authentication key-chain for level 2 PDUs.
    "auth-sendonly-l1": "option",  # Enable/disable level 1 authentication send-only.
    "auth-sendonly-l2": "option",  # Enable/disable level 2 authentication send-only.
    "ignore-lsp-errors": "option",  # Enable/disable ignoring of LSP errors with bad checksums.
    "lsp-gen-interval-l1": "integer",  # Minimum interval for level 1 LSP regenerating.
    "lsp-gen-interval-l2": "integer",  # Minimum interval for level 2 LSP regenerating.
    "lsp-refresh-interval": "integer",  # LSP refresh time in seconds.
    "max-lsp-lifetime": "integer",  # Maximum LSP lifetime in seconds.
    "spf-interval-exp-l1": "user",  # Level 1 SPF calculation delay.
    "spf-interval-exp-l2": "user",  # Level 2 SPF calculation delay.
    "dynamic-hostname": "option",  # Enable/disable dynamic hostname.
    "adjacency-check": "option",  # Enable/disable adjacency check.
    "adjacency-check6": "option",  # Enable/disable IPv6 adjacency check.
    "overload-bit": "option",  # Enable/disable signal other routers not to use us in SPF.
    "overload-bit-suppress": "option",  # Suppress overload-bit for the specific prefixes.
    "overload-bit-on-startup": "integer",  # Overload-bit only temporarily after reboot.
    "default-originate": "option",  # Enable/disable distribution of default route information.
    "default-originate6": "option",  # Enable/disable distribution of default IPv6 route informatio
    "metric-style": "option",  # Use old-style (ISO 10589) or new-style packet formats.
    "redistribute-l1": "option",  # Enable/disable redistribution of level 1 routes into level 2
    "redistribute-l1-list": "string",  # Access-list for route redistribution from l1 to l2.
    "redistribute-l2": "option",  # Enable/disable redistribution of level 2 routes into level 1
    "redistribute-l2-list": "string",  # Access-list for route redistribution from l2 to l1.
    "redistribute6-l1": "option",  # Enable/disable redistribution of level 1 IPv6 routes into le
    "redistribute6-l1-list": "string",  # Access-list for IPv6 route redistribution from l1 to l2.
    "redistribute6-l2": "option",  # Enable/disable redistribution of level 2 IPv6 routes into le
    "redistribute6-l2-list": "string",  # Access-list for IPv6 route redistribution from l2 to l1.
    "isis-net": "string",  # IS-IS net configuration.
    "isis-interface": "string",  # IS-IS interface configuration.
    "summary-address": "string",  # IS-IS summary addresses.
    "summary-address6": "string",  # IS-IS IPv6 summary address.
    "redistribute": "string",  # IS-IS redistribute protocols.
    "redistribute6": "string",  # IS-IS IPv6 redistribution for routing protocols.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "is-type": "IS type.",
    "adv-passive-only": "Enable/disable IS-IS advertisement of passive interfaces only.",
    "adv-passive-only6": "Enable/disable IPv6 IS-IS advertisement of passive interfaces only.",
    "auth-mode-l1": "Level 1 authentication mode.",
    "auth-mode-l2": "Level 2 authentication mode.",
    "auth-password-l1": "Authentication password for level 1 PDUs.",
    "auth-password-l2": "Authentication password for level 2 PDUs.",
    "auth-keychain-l1": "Authentication key-chain for level 1 PDUs.",
    "auth-keychain-l2": "Authentication key-chain for level 2 PDUs.",
    "auth-sendonly-l1": "Enable/disable level 1 authentication send-only.",
    "auth-sendonly-l2": "Enable/disable level 2 authentication send-only.",
    "ignore-lsp-errors": "Enable/disable ignoring of LSP errors with bad checksums.",
    "lsp-gen-interval-l1": "Minimum interval for level 1 LSP regenerating.",
    "lsp-gen-interval-l2": "Minimum interval for level 2 LSP regenerating.",
    "lsp-refresh-interval": "LSP refresh time in seconds.",
    "max-lsp-lifetime": "Maximum LSP lifetime in seconds.",
    "spf-interval-exp-l1": "Level 1 SPF calculation delay.",
    "spf-interval-exp-l2": "Level 2 SPF calculation delay.",
    "dynamic-hostname": "Enable/disable dynamic hostname.",
    "adjacency-check": "Enable/disable adjacency check.",
    "adjacency-check6": "Enable/disable IPv6 adjacency check.",
    "overload-bit": "Enable/disable signal other routers not to use us in SPF.",
    "overload-bit-suppress": "Suppress overload-bit for the specific prefixes.",
    "overload-bit-on-startup": "Overload-bit only temporarily after reboot.",
    "default-originate": "Enable/disable distribution of default route information.",
    "default-originate6": "Enable/disable distribution of default IPv6 route information.",
    "metric-style": "Use old-style (ISO 10589) or new-style packet formats.",
    "redistribute-l1": "Enable/disable redistribution of level 1 routes into level 2.",
    "redistribute-l1-list": "Access-list for route redistribution from l1 to l2.",
    "redistribute-l2": "Enable/disable redistribution of level 2 routes into level 1.",
    "redistribute-l2-list": "Access-list for route redistribution from l2 to l1.",
    "redistribute6-l1": "Enable/disable redistribution of level 1 IPv6 routes into level 2.",
    "redistribute6-l1-list": "Access-list for IPv6 route redistribution from l1 to l2.",
    "redistribute6-l2": "Enable/disable redistribution of level 2 IPv6 routes into level 1.",
    "redistribute6-l2-list": "Access-list for IPv6 route redistribution from l2 to l1.",
    "isis-net": "IS-IS net configuration.",
    "isis-interface": "IS-IS interface configuration.",
    "summary-address": "IS-IS summary addresses.",
    "summary-address6": "IS-IS IPv6 summary address.",
    "redistribute": "IS-IS redistribute protocols.",
    "redistribute6": "IS-IS IPv6 redistribution for routing protocols.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "auth-keychain-l1": {"type": "string", "max_length": 35},
    "auth-keychain-l2": {"type": "string", "max_length": 35},
    "lsp-gen-interval-l1": {"type": "integer", "min": 1, "max": 120},
    "lsp-gen-interval-l2": {"type": "integer", "min": 1, "max": 120},
    "lsp-refresh-interval": {"type": "integer", "min": 1, "max": 65535},
    "max-lsp-lifetime": {"type": "integer", "min": 350, "max": 65535},
    "overload-bit-on-startup": {"type": "integer", "min": 5, "max": 86400},
    "redistribute-l1-list": {"type": "string", "max_length": 35},
    "redistribute-l2-list": {"type": "string", "max_length": 35},
    "redistribute6-l1-list": {"type": "string", "max_length": 35},
    "redistribute6-l2-list": {"type": "string", "max_length": 35},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "isis-net": {
        "id": {
            "type": "integer",
            "help": "ISIS network ID.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "net": {
            "type": "user",
            "help": "IS-IS networks (format = xx.xxxx.  .xxxx.xx.).",
            "default": "",
        },
    },
    "isis-interface": {
        "name": {
            "type": "string",
            "help": "IS-IS interface name.",
            "default": "",
            "max_length": 15,
        },
        "status": {
            "type": "option",
            "help": "Enable/disable interface for IS-IS.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "status6": {
            "type": "option",
            "help": "Enable/disable IPv6 interface for IS-IS.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "network-type": {
            "type": "option",
            "help": "IS-IS interface's network type.",
            "default": "",
            "options": ["broadcast", "point-to-point", "loopback"],
        },
        "circuit-type": {
            "type": "option",
            "help": "IS-IS interface's circuit type.",
            "default": "level-1-2",
            "options": ["level-1-2", "level-1", "level-2"],
        },
        "csnp-interval-l1": {
            "type": "integer",
            "help": "Level 1 CSNP interval.",
            "default": 10,
            "min_value": 1,
            "max_value": 65535,
        },
        "csnp-interval-l2": {
            "type": "integer",
            "help": "Level 2 CSNP interval.",
            "default": 10,
            "min_value": 1,
            "max_value": 65535,
        },
        "hello-interval-l1": {
            "type": "integer",
            "help": "Level 1 hello interval.",
            "default": 10,
            "min_value": 0,
            "max_value": 65535,
        },
        "hello-interval-l2": {
            "type": "integer",
            "help": "Level 2 hello interval.",
            "default": 10,
            "min_value": 0,
            "max_value": 65535,
        },
        "hello-multiplier-l1": {
            "type": "integer",
            "help": "Level 1 multiplier for Hello holding time.",
            "default": 3,
            "min_value": 2,
            "max_value": 100,
        },
        "hello-multiplier-l2": {
            "type": "integer",
            "help": "Level 2 multiplier for Hello holding time.",
            "default": 3,
            "min_value": 2,
            "max_value": 100,
        },
        "hello-padding": {
            "type": "option",
            "help": "Enable/disable padding to IS-IS hello packets.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
        "lsp-interval": {
            "type": "integer",
            "help": "LSP transmission interval (milliseconds).",
            "default": 33,
            "min_value": 1,
            "max_value": 4294967295,
        },
        "lsp-retransmit-interval": {
            "type": "integer",
            "help": "LSP retransmission interval (sec).",
            "default": 5,
            "min_value": 1,
            "max_value": 65535,
        },
        "metric-l1": {
            "type": "integer",
            "help": "Level 1 metric for interface.",
            "default": 10,
            "min_value": 1,
            "max_value": 63,
        },
        "metric-l2": {
            "type": "integer",
            "help": "Level 2 metric for interface.",
            "default": 10,
            "min_value": 1,
            "max_value": 63,
        },
        "wide-metric-l1": {
            "type": "integer",
            "help": "Level 1 wide metric for interface.",
            "default": 10,
            "min_value": 1,
            "max_value": 16777214,
        },
        "wide-metric-l2": {
            "type": "integer",
            "help": "Level 2 wide metric for interface.",
            "default": 10,
            "min_value": 1,
            "max_value": 16777214,
        },
        "auth-password-l1": {
            "type": "password",
            "help": "Authentication password for level 1 PDUs.",
            "max_length": 128,
        },
        "auth-password-l2": {
            "type": "password",
            "help": "Authentication password for level 2 PDUs.",
            "max_length": 128,
        },
        "auth-keychain-l1": {
            "type": "string",
            "help": "Authentication key-chain for level 1 PDUs.",
            "default": "",
            "max_length": 35,
        },
        "auth-keychain-l2": {
            "type": "string",
            "help": "Authentication key-chain for level 2 PDUs.",
            "default": "",
            "max_length": 35,
        },
        "auth-send-only-l1": {
            "type": "option",
            "help": "Enable/disable authentication send-only for level 1 PDUs.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "auth-send-only-l2": {
            "type": "option",
            "help": "Enable/disable authentication send-only for level 2 PDUs.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "auth-mode-l1": {
            "type": "option",
            "help": "Level 1 authentication mode.",
            "default": "password",
            "options": ["md5", "password"],
        },
        "auth-mode-l2": {
            "type": "option",
            "help": "Level 2 authentication mode.",
            "default": "password",
            "options": ["md5", "password"],
        },
        "priority-l1": {
            "type": "integer",
            "help": "Level 1 priority.",
            "default": 64,
            "min_value": 0,
            "max_value": 127,
        },
        "priority-l2": {
            "type": "integer",
            "help": "Level 2 priority.",
            "default": 64,
            "min_value": 0,
            "max_value": 127,
        },
        "mesh-group": {
            "type": "option",
            "help": "Enable/disable IS-IS mesh group.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "mesh-group-id": {
            "type": "integer",
            "help": "Mesh group ID <0-4294967295>, 0: mesh-group blocked.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
    },
    "summary-address": {
        "id": {
            "type": "integer",
            "help": "Summary address entry ID.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "prefix": {
            "type": "ipv4-classnet-any",
            "help": "Prefix.",
            "required": True,
            "default": "0.0.0.0 0.0.0.0",
        },
        "level": {
            "type": "option",
            "help": "Level.",
            "default": "level-2",
            "options": ["level-1-2", "level-1", "level-2"],
        },
    },
    "summary-address6": {
        "id": {
            "type": "integer",
            "help": "Prefix entry ID.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "prefix6": {
            "type": "ipv6-prefix",
            "help": "IPv6 prefix.",
            "required": True,
            "default": "::/0",
        },
        "level": {
            "type": "option",
            "help": "Level.",
            "default": "level-2",
            "options": ["level-1-2", "level-1", "level-2"],
        },
    },
    "redistribute": {
        "protocol": {
            "type": "string",
            "help": "Protocol name.",
            "required": True,
            "default": "",
            "max_length": 35,
        },
        "status": {
            "type": "option",
            "help": "Status.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "metric": {
            "type": "integer",
            "help": "Metric.",
            "default": 0,
            "min_value": 0,
            "max_value": 4261412864,
        },
        "metric-type": {
            "type": "option",
            "help": "Metric type.",
            "default": "internal",
            "options": ["external", "internal"],
        },
        "level": {
            "type": "option",
            "help": "Level.",
            "default": "level-2",
            "options": ["level-1-2", "level-1", "level-2"],
        },
        "routemap": {
            "type": "string",
            "help": "Route map name.",
            "default": "",
            "max_length": 35,
        },
    },
    "redistribute6": {
        "protocol": {
            "type": "string",
            "help": "Protocol name.",
            "required": True,
            "default": "",
            "max_length": 35,
        },
        "status": {
            "type": "option",
            "help": "Enable/disable redistribution.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "metric": {
            "type": "integer",
            "help": "Metric.",
            "default": 0,
            "min_value": 0,
            "max_value": 4261412864,
        },
        "metric-type": {
            "type": "option",
            "help": "Metric type.",
            "default": "internal",
            "options": ["external", "internal"],
        },
        "level": {
            "type": "option",
            "help": "Level.",
            "default": "level-2",
            "options": ["level-1-2", "level-1", "level-2"],
        },
        "routemap": {
            "type": "string",
            "help": "Route map name.",
            "default": "",
            "max_length": 35,
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_IS_TYPE = [
    "level-1-2",
    "level-1",
    "level-2-only",
]
VALID_BODY_ADV_PASSIVE_ONLY = [
    "enable",
    "disable",
]
VALID_BODY_ADV_PASSIVE_ONLY6 = [
    "enable",
    "disable",
]
VALID_BODY_AUTH_MODE_L1 = [
    "password",
    "md5",
]
VALID_BODY_AUTH_MODE_L2 = [
    "password",
    "md5",
]
VALID_BODY_AUTH_SENDONLY_L1 = [
    "enable",
    "disable",
]
VALID_BODY_AUTH_SENDONLY_L2 = [
    "enable",
    "disable",
]
VALID_BODY_IGNORE_LSP_ERRORS = [
    "enable",
    "disable",
]
VALID_BODY_DYNAMIC_HOSTNAME = [
    "enable",
    "disable",
]
VALID_BODY_ADJACENCY_CHECK = [
    "enable",
    "disable",
]
VALID_BODY_ADJACENCY_CHECK6 = [
    "enable",
    "disable",
]
VALID_BODY_OVERLOAD_BIT = [
    "enable",
    "disable",
]
VALID_BODY_OVERLOAD_BIT_SUPPRESS = [
    "external",
    "interlevel",
]
VALID_BODY_DEFAULT_ORIGINATE = [
    "enable",
    "disable",
]
VALID_BODY_DEFAULT_ORIGINATE6 = [
    "enable",
    "disable",
]
VALID_BODY_METRIC_STYLE = [
    "narrow",
    "wide",
    "transition",
    "narrow-transition",
    "narrow-transition-l1",
    "narrow-transition-l2",
    "wide-l1",
    "wide-l2",
    "wide-transition",
    "wide-transition-l1",
    "wide-transition-l2",
    "transition-l1",
    "transition-l2",
]
VALID_BODY_REDISTRIBUTE_L1 = [
    "enable",
    "disable",
]
VALID_BODY_REDISTRIBUTE_L2 = [
    "enable",
    "disable",
]
VALID_BODY_REDISTRIBUTE6_L1 = [
    "enable",
    "disable",
]
VALID_BODY_REDISTRIBUTE6_L2 = [
    "enable",
    "disable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_router_isis_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for router/isis."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_router_isis_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new router/isis object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "is-type" in payload:
        is_valid, error = _validate_enum_field(
            "is-type",
            payload["is-type"],
            VALID_BODY_IS_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "adv-passive-only" in payload:
        is_valid, error = _validate_enum_field(
            "adv-passive-only",
            payload["adv-passive-only"],
            VALID_BODY_ADV_PASSIVE_ONLY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "adv-passive-only6" in payload:
        is_valid, error = _validate_enum_field(
            "adv-passive-only6",
            payload["adv-passive-only6"],
            VALID_BODY_ADV_PASSIVE_ONLY6,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-mode-l1" in payload:
        is_valid, error = _validate_enum_field(
            "auth-mode-l1",
            payload["auth-mode-l1"],
            VALID_BODY_AUTH_MODE_L1,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-mode-l2" in payload:
        is_valid, error = _validate_enum_field(
            "auth-mode-l2",
            payload["auth-mode-l2"],
            VALID_BODY_AUTH_MODE_L2,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-sendonly-l1" in payload:
        is_valid, error = _validate_enum_field(
            "auth-sendonly-l1",
            payload["auth-sendonly-l1"],
            VALID_BODY_AUTH_SENDONLY_L1,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-sendonly-l2" in payload:
        is_valid, error = _validate_enum_field(
            "auth-sendonly-l2",
            payload["auth-sendonly-l2"],
            VALID_BODY_AUTH_SENDONLY_L2,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ignore-lsp-errors" in payload:
        is_valid, error = _validate_enum_field(
            "ignore-lsp-errors",
            payload["ignore-lsp-errors"],
            VALID_BODY_IGNORE_LSP_ERRORS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "dynamic-hostname" in payload:
        is_valid, error = _validate_enum_field(
            "dynamic-hostname",
            payload["dynamic-hostname"],
            VALID_BODY_DYNAMIC_HOSTNAME,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "adjacency-check" in payload:
        is_valid, error = _validate_enum_field(
            "adjacency-check",
            payload["adjacency-check"],
            VALID_BODY_ADJACENCY_CHECK,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "adjacency-check6" in payload:
        is_valid, error = _validate_enum_field(
            "adjacency-check6",
            payload["adjacency-check6"],
            VALID_BODY_ADJACENCY_CHECK6,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "overload-bit" in payload:
        is_valid, error = _validate_enum_field(
            "overload-bit",
            payload["overload-bit"],
            VALID_BODY_OVERLOAD_BIT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "overload-bit-suppress" in payload:
        is_valid, error = _validate_enum_field(
            "overload-bit-suppress",
            payload["overload-bit-suppress"],
            VALID_BODY_OVERLOAD_BIT_SUPPRESS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "default-originate" in payload:
        is_valid, error = _validate_enum_field(
            "default-originate",
            payload["default-originate"],
            VALID_BODY_DEFAULT_ORIGINATE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "default-originate6" in payload:
        is_valid, error = _validate_enum_field(
            "default-originate6",
            payload["default-originate6"],
            VALID_BODY_DEFAULT_ORIGINATE6,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "metric-style" in payload:
        is_valid, error = _validate_enum_field(
            "metric-style",
            payload["metric-style"],
            VALID_BODY_METRIC_STYLE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "redistribute-l1" in payload:
        is_valid, error = _validate_enum_field(
            "redistribute-l1",
            payload["redistribute-l1"],
            VALID_BODY_REDISTRIBUTE_L1,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "redistribute-l2" in payload:
        is_valid, error = _validate_enum_field(
            "redistribute-l2",
            payload["redistribute-l2"],
            VALID_BODY_REDISTRIBUTE_L2,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "redistribute6-l1" in payload:
        is_valid, error = _validate_enum_field(
            "redistribute6-l1",
            payload["redistribute6-l1"],
            VALID_BODY_REDISTRIBUTE6_L1,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "redistribute6-l2" in payload:
        is_valid, error = _validate_enum_field(
            "redistribute6-l2",
            payload["redistribute6-l2"],
            VALID_BODY_REDISTRIBUTE6_L2,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_router_isis_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update router/isis."""
    # Validate enum values using central function
    if "is-type" in payload:
        is_valid, error = _validate_enum_field(
            "is-type",
            payload["is-type"],
            VALID_BODY_IS_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "adv-passive-only" in payload:
        is_valid, error = _validate_enum_field(
            "adv-passive-only",
            payload["adv-passive-only"],
            VALID_BODY_ADV_PASSIVE_ONLY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "adv-passive-only6" in payload:
        is_valid, error = _validate_enum_field(
            "adv-passive-only6",
            payload["adv-passive-only6"],
            VALID_BODY_ADV_PASSIVE_ONLY6,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-mode-l1" in payload:
        is_valid, error = _validate_enum_field(
            "auth-mode-l1",
            payload["auth-mode-l1"],
            VALID_BODY_AUTH_MODE_L1,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-mode-l2" in payload:
        is_valid, error = _validate_enum_field(
            "auth-mode-l2",
            payload["auth-mode-l2"],
            VALID_BODY_AUTH_MODE_L2,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-sendonly-l1" in payload:
        is_valid, error = _validate_enum_field(
            "auth-sendonly-l1",
            payload["auth-sendonly-l1"],
            VALID_BODY_AUTH_SENDONLY_L1,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-sendonly-l2" in payload:
        is_valid, error = _validate_enum_field(
            "auth-sendonly-l2",
            payload["auth-sendonly-l2"],
            VALID_BODY_AUTH_SENDONLY_L2,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ignore-lsp-errors" in payload:
        is_valid, error = _validate_enum_field(
            "ignore-lsp-errors",
            payload["ignore-lsp-errors"],
            VALID_BODY_IGNORE_LSP_ERRORS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "dynamic-hostname" in payload:
        is_valid, error = _validate_enum_field(
            "dynamic-hostname",
            payload["dynamic-hostname"],
            VALID_BODY_DYNAMIC_HOSTNAME,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "adjacency-check" in payload:
        is_valid, error = _validate_enum_field(
            "adjacency-check",
            payload["adjacency-check"],
            VALID_BODY_ADJACENCY_CHECK,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "adjacency-check6" in payload:
        is_valid, error = _validate_enum_field(
            "adjacency-check6",
            payload["adjacency-check6"],
            VALID_BODY_ADJACENCY_CHECK6,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "overload-bit" in payload:
        is_valid, error = _validate_enum_field(
            "overload-bit",
            payload["overload-bit"],
            VALID_BODY_OVERLOAD_BIT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "overload-bit-suppress" in payload:
        is_valid, error = _validate_enum_field(
            "overload-bit-suppress",
            payload["overload-bit-suppress"],
            VALID_BODY_OVERLOAD_BIT_SUPPRESS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "default-originate" in payload:
        is_valid, error = _validate_enum_field(
            "default-originate",
            payload["default-originate"],
            VALID_BODY_DEFAULT_ORIGINATE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "default-originate6" in payload:
        is_valid, error = _validate_enum_field(
            "default-originate6",
            payload["default-originate6"],
            VALID_BODY_DEFAULT_ORIGINATE6,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "metric-style" in payload:
        is_valid, error = _validate_enum_field(
            "metric-style",
            payload["metric-style"],
            VALID_BODY_METRIC_STYLE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "redistribute-l1" in payload:
        is_valid, error = _validate_enum_field(
            "redistribute-l1",
            payload["redistribute-l1"],
            VALID_BODY_REDISTRIBUTE_L1,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "redistribute-l2" in payload:
        is_valid, error = _validate_enum_field(
            "redistribute-l2",
            payload["redistribute-l2"],
            VALID_BODY_REDISTRIBUTE_L2,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "redistribute6-l1" in payload:
        is_valid, error = _validate_enum_field(
            "redistribute6-l1",
            payload["redistribute6-l1"],
            VALID_BODY_REDISTRIBUTE6_L1,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "redistribute6-l2" in payload:
        is_valid, error = _validate_enum_field(
            "redistribute6-l2",
            payload["redistribute6-l2"],
            VALID_BODY_REDISTRIBUTE6_L2,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# ============================================================================

from hfortix_fortios._helpers.metadata import (
    get_field_description as _get_field_description,
    get_field_type as _get_field_type,
    get_field_constraints as _get_field_constraints,
    get_field_default as _get_field_default,
    get_field_options as _get_field_options,
    get_nested_schema as _get_nested_schema,
    get_all_fields as _get_all_fields,
    get_field_metadata as _get_field_metadata,
    validate_field_value as _validate_field_value,
)

# Wrapper functions that bind module-specific data to central functions
def get_field_description(field_name: str) -> str | None:
    """Get description/help text for a field."""
    return _get_field_description(FIELD_DESCRIPTIONS, field_name)

def get_field_type(field_name: str) -> str | None:
    """Get the type of a field."""
    return _get_field_type(FIELD_TYPES, field_name)

def get_field_constraints(field_name: str) -> dict[str, Any] | None:
    """Get constraints for a field (min/max values, string length)."""
    return _get_field_constraints(FIELD_CONSTRAINTS, field_name)

def get_field_default(field_name: str) -> Any | None:
    """Get default value for a field."""
    return _get_field_default(FIELDS_WITH_DEFAULTS, field_name)

def get_field_options(field_name: str) -> list[str] | None:
    """Get valid enum options for a field."""
    return _get_field_options(globals(), field_name)

def get_nested_schema(field_name: str) -> dict[str, Any] | None:
    """Get schema for nested table/list fields."""
    return _get_nested_schema(NESTED_SCHEMAS, field_name)

def get_all_fields() -> list[str]:
    """Get list of all field names."""
    return _get_all_fields(FIELD_TYPES)

def get_field_metadata(field_name: str) -> dict[str, Any] | None:
    """Get complete metadata for a field (type, description, constraints, defaults, options)."""
    return _get_field_metadata(
        FIELD_TYPES, FIELD_DESCRIPTIONS, FIELD_CONSTRAINTS,
        FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS, NESTED_SCHEMAS,
        globals(), field_name
    )

def validate_field_value(field_name: str, value: Any) -> tuple[bool, str | None]:
    """Validate a single field value against its constraints."""
    return _validate_field_value(
        FIELD_TYPES, FIELD_DESCRIPTIONS, FIELD_CONSTRAINTS,
        globals(), field_name, value
    )


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "router/isis",
    "category": "cmdb",
    "api_path": "router/isis",
    "help": "Configure IS-IS.",
    "total_fields": 41,
    "required_fields_count": 0,
    "fields_with_defaults_count": 33,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""Type stubs for FORTIVIEW category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .historical_statistics import HistoricalStatistics, HistoricalStatisticsDictMode, HistoricalStatisticsObjectMode
    from .realtime_proxy_statistics import RealtimeProxyStatistics, RealtimeProxyStatisticsDictMode, RealtimeProxyStatisticsObjectMode
    from .realtime_statistics import RealtimeStatistics, RealtimeStatisticsDictMode, RealtimeStatisticsObjectMode
    from .session import SessionDictMode, SessionObjectMode

__all__ = [
    "HistoricalStatistics",
    "RealtimeProxyStatistics",
    "RealtimeStatistics",
    "FortiviewDictMode",
    "FortiviewObjectMode",
]

class FortiviewDictMode:
    """FORTIVIEW API category for dict response mode.
    
    This class is returned when the client is instantiated with response_mode="dict" (default).
    All endpoints return dict/TypedDict responses by default.
    """
    
    session: SessionDictMode
    historical_statistics: HistoricalStatisticsDictMode
    realtime_proxy_statistics: RealtimeProxyStatisticsDictMode
    realtime_statistics: RealtimeStatisticsDictMode

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize fortiview category with HTTP client."""
        ...


class FortiviewObjectMode:
    """FORTIVIEW API category for object response mode.
    
    This class is returned when the client is instantiated with response_mode="object".
    All endpoints return FortiObject responses by default.
    """
    
    session: SessionObjectMode
    historical_statistics: HistoricalStatisticsObjectMode
    realtime_proxy_statistics: RealtimeProxyStatisticsObjectMode
    realtime_statistics: RealtimeStatisticsObjectMode

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize fortiview category with HTTP client."""
        ...


# Base class for backwards compatibility
class Fortiview:
    """FORTIVIEW API category."""
    
    session: Session
    historical_statistics: HistoricalStatistics
    realtime_proxy_statistics: RealtimeProxyStatistics
    realtime_statistics: RealtimeStatistics

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize fortiview category with HTTP client."""
        ...



"""
Pydantic Models for CMDB - system/dns

Runtime validation models for system/dns configuration.
Generated from FortiOS schema version unknown.

Example Usage:
    >>> from hfortix_fortios.models.cmdb.system.dns import 
    >>>
    >>> # Create with validation
    >>> obj = (
    ...     name="example",
    ... )
    >>>
    >>> # Validation happens automatically
    >>> # ValidationError raised if constraints violated
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Child Table Models
# ============================================================================

class DnsServerHostname(BaseModel):
    """
    Child table model for server-hostname.
    
    DNS server host name list.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    hostname: str = Field(max_length=127, default="", description="DNS server host name list separated by space (maximum 4 domains).")
class DnsDomain(BaseModel):
    """
    Child table model for domain.
    
    Search suffix list for hostname lookup.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    domain: str = Field(max_length=127, default="", description="DNS search domain list separated by space (maximum 8 domains).")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class DnsModel(BaseModel):
    """
    Pydantic model for system/dns configuration.
    
    Configure DNS.
    
    Validation Rules:        - primary: pattern=        - secondary: pattern=        - protocol: pattern=        - ssl_certificate: max_length=35 pattern=        - server_hostname: pattern=        - domain: pattern=        - ip6_primary: pattern=        - ip6_secondary: pattern=        - timeout: min=1 max=10 pattern=        - retry: min=0 max=5 pattern=        - dns_cache_limit: min=0 max=4294967295 pattern=        - dns_cache_ttl: min=60 max=86400 pattern=        - cache_notfound_responses: pattern=        - source_ip: pattern=        - source_ip_interface: max_length=15 pattern=        - root_servers: pattern=        - interface_select_method: pattern=        - interface: max_length=15 pattern=        - vrf_select: min=0 max=511 pattern=        - server_select_method: pattern=        - alt_primary: pattern=        - alt_secondary: pattern=        - log: pattern=        - fqdn_cache_ttl: min=0 max=86400 pattern=        - fqdn_max_refresh: min=3600 max=86400 pattern=        - fqdn_min_refresh: min=10 max=3600 pattern=        - hostname_ttl: min=60 max=86400 pattern=        - hostname_limit: min=0 max=50000 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    primary: str = Field(default="0.0.0.0", description="Primary DNS server IP address.")    
    secondary: str | None = Field(default="0.0.0.0", description="Secondary DNS server IP address.")    
    protocol: list[Protocol] = Field(default="cleartext", description="DNS transport protocols.")    
    ssl_certificate: str | None = Field(max_length=35, default="Fortinet_Factory", description="Name of local certificate for SSL connections.")  # datasource: ['certificate.local.name']    
    server_hostname: list[ServerHostname] = Field(default=None, description="DNS server host name list.")    
    domain: list[Domain] = Field(default=None, description="Search suffix list for hostname lookup.")    
    ip6_primary: str | None = Field(default="::", description="Primary DNS server IPv6 address.")    
    ip6_secondary: str | None = Field(default="::", description="Secondary DNS server IPv6 address.")    
    timeout: int | None = Field(ge=1, le=10, default=5, description="DNS query timeout interval in seconds (1 - 10).")    
    retry: int | None = Field(ge=0, le=5, default=2, description="Number of times to retry (0 - 5).")    
    dns_cache_limit: int | None = Field(ge=0, le=4294967295, default=5000, description="Maximum number of records in the DNS cache.")    
    dns_cache_ttl: int | None = Field(ge=60, le=86400, default=1800, description="Duration in seconds that the DNS cache retains information.")    
    cache_notfound_responses: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable response from the DNS server when a record is not in cache.")    
    source_ip: str | None = Field(default="0.0.0.0", description="IP address used by the DNS server as its source IP.")    
    source_ip_interface: str | None = Field(max_length=15, default="", description="IP address of the specified interface as the source IP address.")  # datasource: ['system.interface.name']    
    root_servers: list[RootServers] = Field(default="", description="Configure up to two preferred servers that serve the DNS root zone (default uses all 13 root servers).")    
    interface_select_method: Literal["auto", "sdwan", "specify"] | None = Field(default="auto", description="Specify how to select outgoing interface to reach server.")    
    interface: str = Field(max_length=15, default="", description="Specify outgoing interface to reach server.")  # datasource: ['system.interface.name']    
    vrf_select: int | None = Field(ge=0, le=511, default=0, description="VRF ID used for connection to server.")    
    server_select_method: Literal["least-rtt", "failover"] | None = Field(default="least-rtt", description="Specify how configured servers are prioritized.")    
    alt_primary: str | None = Field(default="0.0.0.0", description="Alternate primary DNS server. This is not used as a failover DNS server.")    
    alt_secondary: str | None = Field(default="0.0.0.0", description="Alternate secondary DNS server. This is not used as a failover DNS server.")    
    log: Literal["disable", "error", "all"] | None = Field(default="disable", description="Local DNS log setting.")    
    fqdn_cache_ttl: int | None = Field(ge=0, le=86400, default=0, description="FQDN cache time to live in seconds (0 - 86400, default = 0).")    
    fqdn_max_refresh: int | None = Field(ge=3600, le=86400, default=3600, description="FQDN cache maximum refresh time in seconds (3600 - 86400, default = 3600).")    
    fqdn_min_refresh: int | None = Field(ge=10, le=3600, default=60, description="FQDN cache minimum refresh time in seconds (10 - 3600, default = 60).")    
    hostname_ttl: int | None = Field(ge=60, le=86400, default=86400, description="TTL of hostname table entries (60 - 86400).")    
    hostname_limit: int | None = Field(ge=0, le=50000, default=5000, description="Limit of the number of hostname table entries (0 - 50000).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('ssl_certificate')
    @classmethod
    def validate_ssl_certificate(cls, v: Any) -> Any:
        """
        Validate ssl_certificate field.
        
        Datasource: ['certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('source_ip_interface')
    @classmethod
    def validate_source_ip_interface(cls, v: Any) -> Any:
        """
        Validate source_ip_interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('interface')
    @classmethod
    def validate_interface(cls, v: Any) -> Any:
        """
        Validate interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_ssl_certificate_references(self, client: Any) -> list[str]:
        """
        Validate ssl_certificate references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = DnsModel(
            ...     ssl_certificate="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ssl_certificate_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.dns.post(policy.to_fortios_dict())
        """
        errors = []
        
        # Validate scalar field
        value = getattr(self, "ssl_certificate", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Ssl-Certificate '{value}' not found in "
                "certificate/local"
            )        
        return errors    
    async def validate_source_ip_interface_references(self, client: Any) -> list[str]:
        """
        Validate source_ip_interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = DnsModel(
            ...     source_ip_interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_source_ip_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.dns.post(policy.to_fortios_dict())
        """
        errors = []
        
        # Validate scalar field
        value = getattr(self, "source_ip_interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Source-Ip-Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_interface_references(self, client: Any) -> list[str]:
        """
        Validate interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = DnsModel(
            ...     interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.dns.post(policy.to_fortios_dict())
        """
        errors = []
        
        # Validate scalar field
        value = getattr(self, "interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_ssl_certificate_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_source_ip_interface_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_interface_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "DnsModel",    "DnsServerHostname",    "DnsDomain",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-09T09:47:42.129099Z
# ============================================================================


from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_ADMIN: Literal["discovered", "disable", "enable"]
VALID_BODY_FIRMWARE_PROVISION_LATEST: Literal["disable", "once"]
VALID_BODY_OVERRIDE_LED_STATE: Literal["enable", "disable"]
VALID_BODY_LED_STATE: Literal["enable", "disable"]
VALID_BODY_OVERRIDE_WAN_PORT_MODE: Literal["enable", "disable"]
VALID_BODY_WAN_PORT_MODE: Literal["wan-lan", "wan-only"]
VALID_BODY_OVERRIDE_IP_FRAGMENT: Literal["enable", "disable"]
VALID_BODY_IP_FRAGMENT_PREVENTING: Literal["tcp-mss-adjust", "icmp-unreachable"]
VALID_BODY_OVERRIDE_SPLIT_TUNNEL: Literal["enable", "disable"]
VALID_BODY_SPLIT_TUNNELING_ACL_PATH: Literal["tunnel", "local"]
VALID_BODY_SPLIT_TUNNELING_ACL_LOCAL_AP_SUBNET: Literal["enable", "disable"]
VALID_BODY_OVERRIDE_LAN: Literal["enable", "disable"]
VALID_BODY_OVERRIDE_ALLOWACCESS: Literal["enable", "disable"]
VALID_BODY_ALLOWACCESS: Literal["https", "ssh", "snmp"]
VALID_BODY_OVERRIDE_LOGIN_PASSWD_CHANGE: Literal["enable", "disable"]
VALID_BODY_LOGIN_PASSWD_CHANGE: Literal["yes", "default", "no"]
VALID_BODY_OVERRIDE_DEFAULT_MESH_ROOT: Literal["enable", "disable"]
VALID_BODY_DEFAULT_MESH_ROOT: Literal["enable", "disable"]
VALID_BODY_IMAGE_DOWNLOAD: Literal["enable", "disable"]
VALID_BODY_MESH_BRIDGE_ENABLE: Literal["default", "enable", "disable"]
VALID_BODY_PURDUE_LEVEL: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_ADMIN",
    "VALID_BODY_FIRMWARE_PROVISION_LATEST",
    "VALID_BODY_OVERRIDE_LED_STATE",
    "VALID_BODY_LED_STATE",
    "VALID_BODY_OVERRIDE_WAN_PORT_MODE",
    "VALID_BODY_WAN_PORT_MODE",
    "VALID_BODY_OVERRIDE_IP_FRAGMENT",
    "VALID_BODY_IP_FRAGMENT_PREVENTING",
    "VALID_BODY_OVERRIDE_SPLIT_TUNNEL",
    "VALID_BODY_SPLIT_TUNNELING_ACL_PATH",
    "VALID_BODY_SPLIT_TUNNELING_ACL_LOCAL_AP_SUBNET",
    "VALID_BODY_OVERRIDE_LAN",
    "VALID_BODY_OVERRIDE_ALLOWACCESS",
    "VALID_BODY_ALLOWACCESS",
    "VALID_BODY_OVERRIDE_LOGIN_PASSWD_CHANGE",
    "VALID_BODY_LOGIN_PASSWD_CHANGE",
    "VALID_BODY_OVERRIDE_DEFAULT_MESH_ROOT",
    "VALID_BODY_DEFAULT_MESH_ROOT",
    "VALID_BODY_IMAGE_DOWNLOAD",
    "VALID_BODY_MESH_BRIDGE_ENABLE",
    "VALID_BODY_PURDUE_LEVEL",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]